/**
 * 
 * Copyright (c) Microsoft and contributors.  All rights reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * 
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

package com.microsoft.azure.management.network;

import com.microsoft.azure.management.network.models.AzureAsyncOperationResponse;
import com.microsoft.azure.management.network.models.ConnectionResetSharedKey;
import com.microsoft.azure.management.network.models.ConnectionResetSharedKeyPutResponse;
import com.microsoft.azure.management.network.models.ConnectionSharedKey;
import com.microsoft.azure.management.network.models.ConnectionSharedKeyPutResponse;
import com.microsoft.azure.management.network.models.ConnectionSharedKeyResponse;
import com.microsoft.azure.management.network.models.UpdateOperationResponse;
import com.microsoft.azure.management.network.models.VirtualNetworkGatewayConnection;
import com.microsoft.azure.management.network.models.VirtualNetworkGatewayConnectionGetResponse;
import com.microsoft.azure.management.network.models.VirtualNetworkGatewayConnectionListResponse;
import com.microsoft.azure.management.network.models.VirtualNetworkGatewayConnectionPutResponse;
import com.microsoft.windowsazure.core.OperationResponse;
import com.microsoft.windowsazure.exception.ServiceException;

import java.io.IOException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

/**
* The Network Resource Provider API includes operations for managing the
* Virtual network Gateway for your subscription.
*/
public interface VirtualNetworkGatewayConnectionOperations {
    /**
    * The Put VirtualNetworkGatewayConnection operation creates/updates a
    * virtual network gateway connection in the specified resource group
    * through Network resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The name of the
    * virtual network gateway conenction.
    * @param parameters Required. Parameters supplied to the Begin Create or
    * update Virtual Network Gateway connection operation through Network
    * resource provider.
    * @throws IOException Signals that an I/O exception of some sort has
    * occurred. This class is the general class of exceptions produced by
    * failed or interrupted I/O operations.
    * @throws ServiceException Thrown if an unexpected response is found.
    * @return Response for CreateOrUpdateVirtualNetworkGatewayConnection Api
    * servive call
    */
    VirtualNetworkGatewayConnectionPutResponse beginCreateOrUpdating(String resourceGroupName, String virtualNetworkGatewayConnectionName, VirtualNetworkGatewayConnection parameters) throws IOException, ServiceException;
    
    /**
    * The Put VirtualNetworkGatewayConnection operation creates/updates a
    * virtual network gateway connection in the specified resource group
    * through Network resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The name of the
    * virtual network gateway conenction.
    * @param parameters Required. Parameters supplied to the Begin Create or
    * update Virtual Network Gateway connection operation through Network
    * resource provider.
    * @return Response for CreateOrUpdateVirtualNetworkGatewayConnection Api
    * servive call
    */
    Future<VirtualNetworkGatewayConnectionPutResponse> beginCreateOrUpdatingAsync(String resourceGroupName, String virtualNetworkGatewayConnectionName, VirtualNetworkGatewayConnection parameters);
    
    /**
    * The Delete VirtualNetworkGatewayConnection operation deletes the specifed
    * virtual network Gateway connection through Network resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The name of the
    * virtual network gateway connection.
    * @throws IOException Signals that an I/O exception of some sort has
    * occurred. This class is the general class of exceptions produced by
    * failed or interrupted I/O operations.
    * @throws ServiceException Thrown if an unexpected response is found.
    * @return If the resource provide needs to return an error to any
    * operation, it should return the appropriate HTTP error code and a
    * message body as can be seen below.The message should be localized per
    * the Accept-Language header specified in the original request such thatit
    * could be directly be exposed to users
    */
    UpdateOperationResponse beginDeleting(String resourceGroupName, String virtualNetworkGatewayConnectionName) throws IOException, ServiceException;
    
    /**
    * The Delete VirtualNetworkGatewayConnection operation deletes the specifed
    * virtual network Gateway connection through Network resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The name of the
    * virtual network gateway connection.
    * @return If the resource provide needs to return an error to any
    * operation, it should return the appropriate HTTP error code and a
    * message body as can be seen below.The message should be localized per
    * the Accept-Language header specified in the original request such thatit
    * could be directly be exposed to users
    */
    Future<UpdateOperationResponse> beginDeletingAsync(String resourceGroupName, String virtualNetworkGatewayConnectionName);
    
    /**
    * The VirtualNetworkGatewayConnectionResetSharedKey operation resets the
    * virtual network gateway connection shared key for passed virtual network
    * gateway connection in the specified resource group through Network
    * resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The virtual network
    * gateway connection reset shared key Name.
    * @param parameters Required. Parameters supplied to the Begin Reset
    * Virtual Network Gateway connection shared key operation through Network
    * resource provider.
    * @throws IOException Signals that an I/O exception of some sort has
    * occurred. This class is the general class of exceptions produced by
    * failed or interrupted I/O operations.
    * @throws ServiceException Thrown if an unexpected response is found.
    * @return Response for PutVirtualNetworkGatewayConnectionResetSharedKey Api
    * servive call
    */
    ConnectionResetSharedKeyPutResponse beginResetSharedKey(String resourceGroupName, String virtualNetworkGatewayConnectionName, ConnectionResetSharedKey parameters) throws IOException, ServiceException;
    
    /**
    * The VirtualNetworkGatewayConnectionResetSharedKey operation resets the
    * virtual network gateway connection shared key for passed virtual network
    * gateway connection in the specified resource group through Network
    * resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The virtual network
    * gateway connection reset shared key Name.
    * @param parameters Required. Parameters supplied to the Begin Reset
    * Virtual Network Gateway connection shared key operation through Network
    * resource provider.
    * @return Response for PutVirtualNetworkGatewayConnectionResetSharedKey Api
    * servive call
    */
    Future<ConnectionResetSharedKeyPutResponse> beginResetSharedKeyAsync(String resourceGroupName, String virtualNetworkGatewayConnectionName, ConnectionResetSharedKey parameters);
    
    /**
    * The Put VirtualNetworkGatewayConnectionSharedKey operation sets the
    * virtual network gateway connection shared key for passed virtual network
    * gateway connection in the specified resource group through Network
    * resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The virtual network
    * gateway connection name.
    * @param parameters Required. Parameters supplied to the Begin Set Virtual
    * Network Gateway conection Shared key operation throughNetwork resource
    * provider.
    * @throws IOException Signals that an I/O exception of some sort has
    * occurred. This class is the general class of exceptions produced by
    * failed or interrupted I/O operations.
    * @throws ServiceException Thrown if an unexpected response is found.
    * @return Response for PutVirtualNetworkGatewayConnectionSharedKey Api
    * servive call
    */
    ConnectionSharedKeyPutResponse beginSetSharedKey(String resourceGroupName, String virtualNetworkGatewayConnectionName, ConnectionSharedKey parameters) throws IOException, ServiceException;
    
    /**
    * The Put VirtualNetworkGatewayConnectionSharedKey operation sets the
    * virtual network gateway connection shared key for passed virtual network
    * gateway connection in the specified resource group through Network
    * resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The virtual network
    * gateway connection name.
    * @param parameters Required. Parameters supplied to the Begin Set Virtual
    * Network Gateway conection Shared key operation throughNetwork resource
    * provider.
    * @return Response for PutVirtualNetworkGatewayConnectionSharedKey Api
    * servive call
    */
    Future<ConnectionSharedKeyPutResponse> beginSetSharedKeyAsync(String resourceGroupName, String virtualNetworkGatewayConnectionName, ConnectionSharedKey parameters);
    
    /**
    * The CreateOrUpdate Virtual network Gateway connection creates a new or
    * updates an existing virtualnetwork gateway connection through Network
    * resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The name of the
    * virtual network gateway conenction.
    * @param parameters Required. Parameters supplied to the Begin Create or
    * update Virtual Network Gateway connection operation through Network
    * resource provider.
    * @throws InterruptedException Thrown when a thread is waiting, sleeping,
    * or otherwise occupied, and the thread is interrupted, either before or
    * during the activity. Occasionally a method may wish to test whether the
    * current thread has been interrupted, and if so, to immediately throw
    * this exception. The following code can be used to achieve this effect:
    * @throws ExecutionException Thrown when attempting to retrieve the result
    * of a task that aborted by throwing an exception. This exception can be
    * inspected using the Throwable.getCause() method.
    * @throws IOException Thrown if there was an error setting up tracing for
    * the request.
    * @return The response body contains the status of the specified
    * asynchronous operation, indicating whether it has succeeded, is
    * inprogress, or has failed. Note that this status is distinct from the
    * HTTP status code returned for the Get Operation Status operation itself.
    * If the asynchronous operation succeeded, the response body includes the
    * HTTP status code for the successful request. If the asynchronous
    * operation failed, the response body includes the HTTP status code for
    * the failed request and error information regarding the failure.
    */
    AzureAsyncOperationResponse createOrUpdate(String resourceGroupName, String virtualNetworkGatewayConnectionName, VirtualNetworkGatewayConnection parameters) throws InterruptedException, ExecutionException, IOException;
    
    /**
    * The CreateOrUpdate Virtual network Gateway connection creates a new or
    * updates an existing virtualnetwork gateway connection through Network
    * resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The name of the
    * virtual network gateway conenction.
    * @param parameters Required. Parameters supplied to the Begin Create or
    * update Virtual Network Gateway connection operation through Network
    * resource provider.
    * @return The response body contains the status of the specified
    * asynchronous operation, indicating whether it has succeeded, is
    * inprogress, or has failed. Note that this status is distinct from the
    * HTTP status code returned for the Get Operation Status operation itself.
    * If the asynchronous operation succeeded, the response body includes the
    * HTTP status code for the successful request. If the asynchronous
    * operation failed, the response body includes the HTTP status code for
    * the failed request and error information regarding the failure.
    */
    Future<AzureAsyncOperationResponse> createOrUpdateAsync(String resourceGroupName, String virtualNetworkGatewayConnectionName, VirtualNetworkGatewayConnection parameters);
    
    /**
    * The Delete VirtualNetworkGatewayConnection operation deletes the specifed
    * virtual network Gateway connection through Network resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The name of the
    * virtual network gateway connection.
    * @throws InterruptedException Thrown when a thread is waiting, sleeping,
    * or otherwise occupied, and the thread is interrupted, either before or
    * during the activity. Occasionally a method may wish to test whether the
    * current thread has been interrupted, and if so, to immediately throw
    * this exception. The following code can be used to achieve this effect:
    * @throws ExecutionException Thrown when attempting to retrieve the result
    * of a task that aborted by throwing an exception. This exception can be
    * inspected using the Throwable.getCause() method.
    * @throws IOException Thrown if there was an error setting up tracing for
    * the request.
    * @return A standard service response including an HTTP status code and
    * request ID.
    */
    OperationResponse delete(String resourceGroupName, String virtualNetworkGatewayConnectionName) throws InterruptedException, ExecutionException, IOException;
    
    /**
    * The Delete VirtualNetworkGatewayConnection operation deletes the specifed
    * virtual network Gateway connection through Network resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The name of the
    * virtual network gateway connection.
    * @return A standard service response including an HTTP status code and
    * request ID.
    */
    Future<OperationResponse> deleteAsync(String resourceGroupName, String virtualNetworkGatewayConnectionName);
    
    /**
    * The Get VirtualNetworkGatewayConnection operation retrieves information
    * about the specified virtual network gateway connection through Network
    * resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The name of the
    * virtual network gateway connection.
    * @throws IOException Signals that an I/O exception of some sort has
    * occurred. This class is the general class of exceptions produced by
    * failed or interrupted I/O operations.
    * @throws ServiceException Thrown if an unexpected response is found.
    * @return Response for GetVitualNetworkGatewayConnection Api servive call.
    */
    VirtualNetworkGatewayConnectionGetResponse get(String resourceGroupName, String virtualNetworkGatewayConnectionName) throws IOException, ServiceException;
    
    /**
    * The Get VirtualNetworkGatewayConnection operation retrieves information
    * about the specified virtual network gateway connection through Network
    * resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The name of the
    * virtual network gateway connection.
    * @return Response for GetVitualNetworkGatewayConnection Api servive call.
    */
    Future<VirtualNetworkGatewayConnectionGetResponse> getAsync(String resourceGroupName, String virtualNetworkGatewayConnectionName);
    
    /**
    * The Get VirtualNetworkGatewayConnectionSharedKey operation retrieves
    * information about the specified virtual network gateway connection
    * shared key through Network resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param connectionSharedKeyName Required. The virtual network gateway
    * connection shared key name.
    * @throws IOException Signals that an I/O exception of some sort has
    * occurred. This class is the general class of exceptions produced by
    * failed or interrupted I/O operations.
    * @throws ServiceException Thrown if an unexpected response is found.
    * @return Response for CheckConnectionSharedKey Api servive call
    */
    ConnectionSharedKeyResponse getSharedKey(String resourceGroupName, String connectionSharedKeyName) throws IOException, ServiceException;
    
    /**
    * The Get VirtualNetworkGatewayConnectionSharedKey operation retrieves
    * information about the specified virtual network gateway connection
    * shared key through Network resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param connectionSharedKeyName Required. The virtual network gateway
    * connection shared key name.
    * @return Response for CheckConnectionSharedKey Api servive call
    */
    Future<ConnectionSharedKeyResponse> getSharedKeyAsync(String resourceGroupName, String connectionSharedKeyName);
    
    /**
    * The List VirtualNetworkGatewayConnections operation retrieves all the
    * virtual network gateways connections created.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @throws IOException Signals that an I/O exception of some sort has
    * occurred. This class is the general class of exceptions produced by
    * failed or interrupted I/O operations.
    * @throws ServiceException Thrown if an unexpected response is found.
    * @return Response for ListVirtualNetworkGatewayConnections Api service call
    */
    VirtualNetworkGatewayConnectionListResponse list(String resourceGroupName) throws IOException, ServiceException;
    
    /**
    * The List VirtualNetworkGatewayConnections operation retrieves all the
    * virtual network gateways connections created.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @return Response for ListVirtualNetworkGatewayConnections Api service call
    */
    Future<VirtualNetworkGatewayConnectionListResponse> listAsync(String resourceGroupName);
    
    /**
    * The Reset VirtualNetworkGatewayConnectionSharedKey operation resets the
    * virtual network gateway connection shared key for passed virtual network
    * gateway connection in the specified resource group through Network
    * resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The virtual network
    * gateway connection name.
    * @param parameters Required. Parameters supplied to the Begin Reset
    * Virtual Network Gateway connection shared key operation through Network
    * resource provider.
    * @throws InterruptedException Thrown when a thread is waiting, sleeping,
    * or otherwise occupied, and the thread is interrupted, either before or
    * during the activity. Occasionally a method may wish to test whether the
    * current thread has been interrupted, and if so, to immediately throw
    * this exception. The following code can be used to achieve this effect:
    * @throws ExecutionException Thrown when attempting to retrieve the result
    * of a task that aborted by throwing an exception. This exception can be
    * inspected using the Throwable.getCause() method.
    * @throws IOException Thrown if there was an error setting up tracing for
    * the request.
    * @return The response body contains the status of the specified
    * asynchronous operation, indicating whether it has succeeded, is
    * inprogress, or has failed. Note that this status is distinct from the
    * HTTP status code returned for the Get Operation Status operation itself.
    * If the asynchronous operation succeeded, the response body includes the
    * HTTP status code for the successful request. If the asynchronous
    * operation failed, the response body includes the HTTP status code for
    * the failed request and error information regarding the failure.
    */
    AzureAsyncOperationResponse resetSharedKey(String resourceGroupName, String virtualNetworkGatewayConnectionName, ConnectionResetSharedKey parameters) throws InterruptedException, ExecutionException, IOException;
    
    /**
    * The Reset VirtualNetworkGatewayConnectionSharedKey operation resets the
    * virtual network gateway connection shared key for passed virtual network
    * gateway connection in the specified resource group through Network
    * resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The virtual network
    * gateway connection name.
    * @param parameters Required. Parameters supplied to the Begin Reset
    * Virtual Network Gateway connection shared key operation through Network
    * resource provider.
    * @return The response body contains the status of the specified
    * asynchronous operation, indicating whether it has succeeded, is
    * inprogress, or has failed. Note that this status is distinct from the
    * HTTP status code returned for the Get Operation Status operation itself.
    * If the asynchronous operation succeeded, the response body includes the
    * HTTP status code for the successful request. If the asynchronous
    * operation failed, the response body includes the HTTP status code for
    * the failed request and error information regarding the failure.
    */
    Future<AzureAsyncOperationResponse> resetSharedKeyAsync(String resourceGroupName, String virtualNetworkGatewayConnectionName, ConnectionResetSharedKey parameters);
    
    /**
    * The Put VirtualNetworkGatewayConnectionSharedKey operation sets the
    * virtual network gateway connection shared key for passed virtual network
    * gateway connection in the specified resource group through Network
    * resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The virtual network
    * gateway connection name.
    * @param parameters Required. Parameters supplied to the Begin Create or
    * update Virtual Network Gateway connection shared key operation through
    * Network resource provider.
    * @throws InterruptedException Thrown when a thread is waiting, sleeping,
    * or otherwise occupied, and the thread is interrupted, either before or
    * during the activity. Occasionally a method may wish to test whether the
    * current thread has been interrupted, and if so, to immediately throw
    * this exception. The following code can be used to achieve this effect:
    * @throws ExecutionException Thrown when attempting to retrieve the result
    * of a task that aborted by throwing an exception. This exception can be
    * inspected using the Throwable.getCause() method.
    * @throws IOException Thrown if there was an error setting up tracing for
    * the request.
    * @return The response body contains the status of the specified
    * asynchronous operation, indicating whether it has succeeded, is
    * inprogress, or has failed. Note that this status is distinct from the
    * HTTP status code returned for the Get Operation Status operation itself.
    * If the asynchronous operation succeeded, the response body includes the
    * HTTP status code for the successful request. If the asynchronous
    * operation failed, the response body includes the HTTP status code for
    * the failed request and error information regarding the failure.
    */
    AzureAsyncOperationResponse setSharedKey(String resourceGroupName, String virtualNetworkGatewayConnectionName, ConnectionSharedKey parameters) throws InterruptedException, ExecutionException, IOException;
    
    /**
    * The Put VirtualNetworkGatewayConnectionSharedKey operation sets the
    * virtual network gateway connection shared key for passed virtual network
    * gateway connection in the specified resource group through Network
    * resource provider.
    *
    * @param resourceGroupName Required. The name of the resource group.
    * @param virtualNetworkGatewayConnectionName Required. The virtual network
    * gateway connection name.
    * @param parameters Required. Parameters supplied to the Begin Create or
    * update Virtual Network Gateway connection shared key operation through
    * Network resource provider.
    * @return The response body contains the status of the specified
    * asynchronous operation, indicating whether it has succeeded, is
    * inprogress, or has failed. Note that this status is distinct from the
    * HTTP status code returned for the Get Operation Status operation itself.
    * If the asynchronous operation succeeded, the response body includes the
    * HTTP status code for the successful request. If the asynchronous
    * operation failed, the response body includes the HTTP status code for
    * the failed request and error information regarding the failure.
    */
    Future<AzureAsyncOperationResponse> setSharedKeyAsync(String resourceGroupName, String virtualNetworkGatewayConnectionName, ConnectionSharedKey parameters);
}
